@@ -13,8 +13,6 @@ import platform
 
 from .encoding import DEFAULT_ENCODING
 
-def no_code(x, encoding=None):
-    return x
 
 def decode(s, encoding=None):
     encoding = encoding or DEFAULT_ENCODING
@@ -44,7 +42,7 @@ def buffer_to_bytes(buf):
 def _modify_str_or_docstring(str_change_func):
     @functools.wraps(str_change_func)
     def wrapper(func_or_str):
-        if isinstance(func_or_str, string_types):
+        if isinstance(func_or_str, (str,)):
             func = None
             doc = func_or_str
         else:
@@ -66,17 +64,12 @@ def safe_unicode(e):
     safe to call unicode() on.
     """
     try:
-        return unicode_type(e)
+        return str(e)
     except UnicodeError:
         pass
 
     try:
-        return str_to_unicode(str(e))
-    except UnicodeError:
-        pass
-
-    try:
-        return str_to_unicode(repr(e))
+        return repr(e)
     except UnicodeError:
         pass
 
@@ -156,16 +149,6 @@ def input(prompt=''):
 builtin_mod_name = "builtins"
 import builtins as builtin_mod
 
-str_to_unicode = no_code
-unicode_to_str = no_code
-str_to_bytes = encode
-bytes_to_str = decode
-cast_bytes_py2 = no_code
-cast_unicode_py2 = no_code
-buffer_to_bytes_py2 = no_code
-
-string_types = (str,)
-unicode_type = str
 
 which = shutil.which
 
@@ -174,9 +157,6 @@ def isidentifier(s, dotted=False):
         return all(isidentifier(a) for a in s.split("."))
     return s.isidentifier()
 
-xrange = range
-def iteritems(d): return iter(d.items())
-def itervalues(d): return iter(d.values())
 getcwd = os.getcwd
 
 MethodType = types.MethodType
@@ -201,50 +181,14 @@ def u_format(s):
     Accepts a string or a function, so it can be used as a decorator."""
     return s.format(u='')
 
-def get_closure(f):
-    """Get a function's closure attribute"""
-    return f.__closure__
-
 
 PY2 = not PY3
 PYPY = platform.python_implementation() == "PyPy"
 
+# Cython still rely on that as a Dec 28 2019
+# See https://github.com/cython/cython/pull/3291 and
+# https://github.com/ipython/ipython/issues/12068
+def no_code(x, encoding=None):
+        return x
+unicode_to_str = cast_bytes_py2 = no_code
 
-def annotate(**kwargs):
-    """Python 3 compatible function annotation for Python 2."""
-    if not kwargs:
-        raise ValueError('annotations must be provided as keyword arguments')
-    def dec(f):
-        if hasattr(f, '__annotations__'):
-            for k, v in kwargs.items():
-                f.__annotations__[k] = v
-        else:
-            f.__annotations__ = kwargs
-        return f
-    return dec
-
-
-# Parts below taken from six:
-# Copyright (c) 2010-2013 Benjamin Peterson
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
-def with_metaclass(meta, *bases):
-    """Create a base class with a metaclass."""
-    return meta("_NewBase", bases, {})
